package me.unurled.sacredrealms.sr.components.player;

import static me.unurled.sacredrealms.sr.utils.Logger.warn;

import com.google.gson.annotations.Expose;
import java.util.HashMap;
import java.util.List;
import java.util.UUID;
import me.unurled.sacredrealms.sr.components.attributes.Attribute;
import org.bukkit.Bukkit;
import org.bukkit.entity.Player;
import org.bukkit.inventory.Inventory;
import org.bukkit.inventory.ItemStack;
import org.bukkit.inventory.meta.ItemMeta;
import org.bukkit.persistence.PersistentDataType;
import org.bukkit.potion.PotionEffect;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

public class SRPlayer {

  @Expose
  private final UUID uuid;

  @Expose
  private Inventory inventory;

  @Expose
  private final HashMap<Attribute, Double> attributes;

  @Expose
  private final List<PotionEffect> potionEffects;

  public SRPlayer(@NotNull Player player) {
    this.uuid = player.getUniqueId();
    this.inventory = player.getInventory();
    this.attributes = new HashMap<>();
    this.potionEffects = (List<PotionEffect>) player.getActivePotionEffects();
  }

  @NotNull
  public UUID getUuid() {
    return uuid;
  }

  public void setInventory(Inventory inventory) {
    this.inventory = inventory;
  }

  @NotNull
  public Inventory getInventory() {
    return inventory;
  }

  @NotNull
  public HashMap<Attribute, Double> getAttributes() {
    return attributes;
  }

  public double getAttribute(@NotNull Attribute attribute) {
    return getAttribute(attribute, attribute.getDefaultValue());
  }

  @NotNull
  public Double getAttribute(@NotNull Attribute attribute, @NotNull Double defaultValue) {
    return attributes.getOrDefault(attribute, defaultValue);
  }

  public void setAttribute(@NotNull Attribute attribute, @NotNull Double value) {
    attributes.put(attribute, value);
  }

  public void removeAttribute(@NotNull Attribute attribute) {
    attributes.remove(attribute);
  }

  @Nullable
  public Object getHandItemAttribute(@NotNull Attribute attribute) {
    Player p = Bukkit.getPlayer(uuid);
    if (p == null) {
      return null;
    }
    Object o = p.getInventory()
        .getItemInMainHand()
        .getItemMeta()
        .getPersistentDataContainer()
        .get(attribute.getKey(), attribute.getType());
    if (o == null) {
      setHandItemAttribute(attribute, attribute.getDefaultValue());
      return attribute.getDefaultValue();
    }
    return o;
  }

  public void setHandItemAttribute(@NotNull Attribute attribute, @NotNull Double value) {
    Player p = Bukkit.getPlayer(uuid);
    if (p == null) {
      return;
    }
    if (checkIfValueIsOutOfBoundsForAttribute(attribute, value, p))
      return;
    ItemStack i = p.getInventory().getItemInMainHand();
    ItemMeta im = i.getItemMeta();
    im.getPersistentDataContainer()
        .set(attribute.getKey(), PersistentDataType.DOUBLE, value);
    im.lore();
  }

  private boolean checkIfValueIsOutOfBoundsForAttribute(@NotNull Attribute attribute,
      @NotNull Double value,
      Player p) {
    if (!attribute.getType().getComplexType().isInstance(value)) {
      return true;
    }
    if (attribute.getType().equals(PersistentDataType.DOUBLE)) {
      double d = (double) value;
      if (d < attribute.getMinValue() || d > attribute.getMaxValue()) {
        warn(
            "(Player "
                + p.getName()
                + ") Value "
                + d
                + " for attribute "
                + attribute.getID()
                + " is out of bounds. Min: "
                + attribute.getMinValue()
                + " Max: "
                + attribute.getMaxValue()
                + " Value: "
                + d);
        return true;
      }
    }
    return false;
  }

  public void removeHandItemAttribute(@NotNull Attribute attribute) {
    Player p = Bukkit.getPlayer(uuid);
    if (p == null) {
      return;
    }
    ItemStack i = p.getInventory().getItemInMainHand();
    ItemMeta im = i.getItemMeta();
    im.getPersistentDataContainer().remove(attribute.getKey());
    i.setItemMeta(im);
  }

  public void addPotionEffect(@NotNull PotionEffect effect) {
    potionEffects.add(effect);
  }

  public void removePotionEffect(@NotNull PotionEffect effect) {
    potionEffects.remove(effect);
  }

  public void clearPotionEffects() {
    potionEffects.clear();
  }

  public void setPotionEffects(@NotNull List<PotionEffect> effects) {
    potionEffects.clear();
    potionEffects.addAll(effects);
  }

  @NotNull
  public List<PotionEffect> getPotionEffects() {
    return potionEffects;
  }
}
